@@ -3,8 +3,10 @@
 
 ##' @title Plot of labels indicating adjacent groups
 ##' @param zargs argument list as passed from zenplot()
-##' @param glabs group labels being indexed by the plot variables;
+##' @param glabs group labels being indexed by the plot variables
+##'        (and thus of length as the number of variables);
 ##'        if NULL then they are determined with extract_2d()
+##' @param sep group label separator
 ##' @param loc (x,y)-location in [0,1]^2; 0 corresponds to left, 1 to right (in
 ##'        the direction of the path)
 ##' @param draw logical indicating whether drawing should take place
@@ -14,7 +16,7 @@
 ##' @note For performance reasons (avoiding having to call extract_2d() twice),
 ##'       'glabs' is an extra argument
 group_2d_grid <- function(zargs,
-                          glabs, loc = c(0.5, 0.5),
+                          glabs = NULL, sep = "\n", loc = c(0.5, 0.5),
                           draw = FALSE, ...)
 {
     check_zargs(zargs, "turns", "vars", "num", "ispace")
@@ -23,7 +25,14 @@ group_2d_grid <- function(zargs,
     num <- zargs$num
     ii <- range(vars[num,]) # variable index
     ii <- if(turns[num-1] == "u" || turns[num] == "u") rev(ii) else ii
-    labs <- paste0(glabs[ii], collapse = "\n") # labels (in the correct order for displaying the group change)
+    if(is.null(glabs)) {
+        glabs <- extract_2d(zargs)$glabs
+    } else {
+        len.groups <- length(unlist(zargs$x, recursive = FALSE))
+        if(length(glabs) != len.groups)
+            stop("length(glabs) has to equal the number ",len.groups," of variables in all groups together; consider rep()")
+    }
+    labs <- paste0(glabs[ii], collapse = sep) # labels (in the correct order for displaying the group change)
     ## Plotting
     vp <- vport(zargs$ispace)
     res <- textGrob(label = labs, x = loc[1], y = loc[2],
@@ -57,10 +66,9 @@ points_2d_grid <- function(zargs,
     r <- extract_2d(zargs)
     xlim <- r$xlim
     ylim <- r$ylim
-    x <- r$x
-    y <- r$y
+    x <- as.matrix(r$x) # for pointsGrob()
+    y <- as.matrix(r$y)
     same.group <- r$same.group
-    glabs <- r$glabsx
     check_zargs(zargs, "ispace")
     res <- if(same.group) {
         vp <- vport(zargs$ispace, xlim = xlim, ylim = ylim)
@@ -110,7 +118,91 @@ points_2d_grid <- function(zargs,
         },
         stop("Wrong 'type'"))
     } else {
-        args <- c(list(zargs = zargs, glabs = glabs), group...)
+        args <- c(list(zargs = zargs), group...)
+        do.call(group_2d_grid, args)
+    }
+    if(draw) grid.draw(res)
+    invisible(res)
+}
+
+##' @title Q-Q plot in 2d (two data sets against each other)
+##' @param zargs argument list as passed from zenplot()
+##' @param do.line logical indicating whether a line is drawn (through both
+##'        empirical c(0.25, 0.75)-quantiles)
+##' @param lines... additional arguments passed to lines()
+##' @param pch plot symbol
+##' @param size size of the plot symbol
+##' @param box logical indicating whether a box should be drawn
+##' @param box.width width of the box
+##' @param box.height height of the box
+##' @param group... list of arguments passed to group_2d_grid (or NULL)
+##' @param draw logical indicating whether drawing should take place
+##' @param ... additional arguments passed to gpar()
+##' @return grob (invisibly)
+##' @author Marius Hofert and Wayne Oldford
+##' @note - line iff both margins are of the same *type*
+##'       - The default point size was chosen to match the default of graphics
+qq_2d_grid <- function(zargs,
+                       do.line = TRUE, lines... = NULL, pch = NULL, size = 0.02,
+                       box = FALSE, box.width = 1, box.height = 1,
+                       group... = list(cex = 0.66), draw = FALSE, ...)
+{
+    r <- extract_2d(zargs)
+    xlim <- r$xlim
+    ylim <- r$ylim
+    x <- r$x
+    y <- r$y
+    same.group <- r$same.group
+    check_zargs(zargs, "ispace")
+    res <- if(same.group) {
+        vp <- vport(zargs$ispace, xlim = xlim, ylim = ylim)
+        ## Calculation (see qqplot())
+        sx <- sort(x)
+        sy <- sort(y)
+        lenx <- length(sx)
+        leny <- length(sy)
+        if (leny < lenx)
+            sx <- approx(1L:lenx, sx, n = leny)$y
+        if (leny > lenx)
+            sy <- approx(1L:leny, sy, n = lenx)$y
+        ## Plot
+        if(is.null(pch)) pch <- 21
+        gPoints <- pointsGrob(x = sx, y = sy, pch = pch,
+                              size = unit(size, units = "npc"),
+                              default.units = "native",
+                              name = "points_2d", gp = gpar(...), vp = vp) # Q-Q plot
+        groblist <- list(gPoints)
+        if(do.line) {
+            qx <- quantile(x, probs = c(0.25, 0.75), na.rm = TRUE, names = FALSE)
+            qy <- quantile(y, probs = c(0.25, 0.75), na.rm = TRUE, names = FALSE)
+            slope <- diff(qy) / diff(qx)
+            intercept <- qy[1] - qx[1] * slope
+            ## We can't just plot that as there is no abline() in grid. Evaluating
+            ## the line at xlim and corresponding y-values slope * xvals + intercept
+            ## could lie very well outside the plot region.
+            ## We solve this here *brute force*
+            xvals <- seq(xlim[1], xlim[2], length.out = 1024)
+            yvals <- slope * xvals + intercept
+            ok <- (xlim[1] <= xvals) & (xvals <= xlim[2]) &
+                  (ylim[1] <= yvals) & (yvals <= ylim[2])
+            vals <- cbind(xvals, yvals)[ok, ]
+            x0.x1 <- c(vals[1,1], vals[nrow(vals),1])
+            y0.y1 <- c(vals[1,2], vals[nrow(vals),2])
+            gLines <- linesGrob(x = x0.x1, y = y0.y1,
+                                default.units = "native", name = "lines_2d",
+                                gp = gpar(...), vp = vp) # Q-Q line
+            groblist <- c(list(gLines), groblist) # append to list
+        }
+        if(box) {
+            gBox <- rectGrob(x = 0.5, y = 0.5,
+                             width = box.width, height = box.height, # => plotting outside of viewport (space has been reserved by default ispace)
+                             just = "centre", default.units = "npc",
+                             name = "box_2d", gp = gpar(...), vp = vp)
+            groblist <- c(list(gBox), groblist) # append to list
+        }
+        gTree(children = do.call(gList, groblist))
+    } else {
+        args <- c(list(zargs = zargs), group...)
         do.call(group_2d_grid, args)
     }
     if(draw) grid.draw(res)
@@ -150,7 +242,6 @@ density_2d_grid <- function(zargs,
     x <- r$x
     y <- r$y
     same.group <- r$same.group
-    glabs <- r$glabsx
     check_zargs(zargs, "ispace")
     res <- if(same.group) {
         data <- na.omit(data.frame(x, y))
@@ -202,7 +293,7 @@ density_2d_grid <- function(zargs,
             gTree(children = do.call(gList, args = contourGrobs))
         }
     } else {
-        args <- c(list(zargs = zargs, glabs = glabs), group...)
+        args <- c(list(zargs = zargs), group...)
         do.call(group_2d_grid, args)
     }
     if(draw) grid.draw(res)
@@ -231,7 +322,6 @@ axes_2d_grid <- function(zargs,
     x <- r$x
     y <- r$y
     same.group <- r$same.group
-    glabs <- r$glabsx
     check_zargs(zargs, "ispace")
     res <- if(same.group) {
         vp <- vport(zargs$ispace, xlim = xlim, ylim = ylim, x = x, y = y)
@@ -249,7 +339,7 @@ axes_2d_grid <- function(zargs,
                             gp = gpar(...), vp = vp) # y axis
         gTree(children = gList(x.grob, y.grob)) # create a single grob
     } else {
-        args <- c(list(zargs = zargs, glabs = glabs), group...)
+        args <- c(list(zargs = zargs), group...)
         do.call(group_2d_grid, args)
     }
     if(draw) grid.draw(res)
@@ -272,7 +362,6 @@ arrow_2d_grid <- function(zargs,
 {
     r <- extract_2d(zargs)
     same.group <- r$same.group
-    glabs <- r$glabs
     check_zargs(zargs, "num", "turns", "ispace")
     turn.out <- zargs$turns[zargs$num]
     res <- if(same.group) {
@@ -284,7 +373,7 @@ arrow_2d_grid <- function(zargs,
         linesGrob(x = arr[1,], y = arr[2,], default.units = "npc",
                   name = "arrow_2d", gp = gpar(...), vp = vp)
     } else {
-        args <- c(list(zargs = zargs, glabs = glabs), group...)
+        args <- c(list(zargs = zargs), group...)
         do.call(group_2d_grid, args)
     }
     if(draw) grid.draw(res)
@@ -307,7 +396,6 @@ rect_2d_grid <- function(zargs,
 {
     r <- extract_2d(zargs)
     same.group <- r$same.group
-    glabs <- r$glabs
     check_zargs(zargs, "ispace")
     res <- if(same.group) {
         vp <- vport(zargs$ispace)
@@ -315,7 +403,7 @@ rect_2d_grid <- function(zargs,
                  default.units = "npc", name = "rect_2d",
                  gp = gpar(...), vp = vp)
     } else {
-        args <- c(list(zargs = zargs, glabs = glabs), group...)
+        args <- c(list(zargs = zargs), group...)
         do.call(group_2d_grid, args)
     }
     if(draw) grid.draw(res)
@@ -347,7 +435,6 @@ label_2d_grid <- function(zargs,
     r <- extract_2d(zargs)
     same.group <- r$same.group
     vlabs <- r$vlabs
-    glabs <- r$glabs
     check_zargs(zargs, "vars", "num", "ispace")
     vars <- zargs$vars
     num <- zargs$num
@@ -370,7 +457,7 @@ label_2d_grid <- function(zargs,
             gTree(children = gList(gText))
         }
     } else {
-        args <- c(list(zargs = zargs, glabs = glabs), group...)
+        args <- c(list(zargs = zargs), group...)
         do.call(group_2d_grid, args)
     }
     if(draw) grid.draw(res)
